@@ -28,8 +28,12 @@
  */
 
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
+#include "gdl-i18n.h"
 #include <string.h>
-#include <libgnome/gnome-macros.h>
 #include <gdk/gdkkeysyms.h>
 
 #include "gdl-tools.h"
@@ -40,6 +44,7 @@
 #include "gdl-dock-paned.h"
 #include "gdl-dock-tablabel.h"
 #include "gdl-dock-placeholder.h"
+#include "gdl-dock-master.h"
 #include "libgdltypebuiltins.h"
 #include "libgdlmarshal.h"
 
@@ -119,6 +124,9 @@ static void  gdl_dock_item_hide_cb       (GtkWidget   *widget,
 static void  gdl_dock_item_lock_cb       (GtkWidget   *widget,
                                           GdlDockItem *item);
 
+static void  gdl_dock_item_unlock_cb     (GtkWidget   *widget,
+                                          GdlDockItem *item);
+
 static void  gdl_dock_item_showhide_grip (GdlDockItem *item);
 
 static void  gdl_dock_item_real_set_orientation (GdlDockItem    *item,
@@ -138,7 +146,6 @@ enum {
     PROP_ORIENTATION,
     PROP_RESIZE,
     PROP_BEHAVIOR,
-    PROP_GRIP_SIZE,
     PROP_LOCKED,
     PROP_PREFERRED_WIDTH,
     PROP_PREFERRED_HEIGHT
@@ -153,13 +160,8 @@ enum {
 
 static guint gdl_dock_item_signals [LAST_SIGNAL] = { 0 };
 
-#define DEFAULT_GRIP_SIZE  10
-#define GDL_DOCK_ITEM_NOT_LOCKED(item) !((item)->behavior & GDL_DOCK_ITEM_BEH_LOCKED)
 #define GDL_DOCK_ITEM_GRIP_SHOWN(item) \
-    (GDL_DOCK_ITEM_HAS_GRIP (item) && \
-     GDL_DOCK_ITEM_NOT_LOCKED (item) && \
-     (item)->_priv->grip_shown)
-
+    (GDL_DOCK_ITEM_HAS_GRIP (item)) 
 
 struct _GdlDockItemPrivate {
     GtkWidget *menu;
@@ -178,14 +180,6 @@ struct _GdlDockItemPrivate {
     gint       start_x, start_y;
 };
 
-typedef struct _GdlDockItemMenuData GdlDockItemMenuData;
-
-struct _GdlDockItemMenuData {
-    GtkWidget *dock, *undock;
-    GtkWidget *hide, *lock;
-    GtkWidget *first_option;
-};
-
 /* FIXME: implement the rest of the behaviors */
 
 #define SPLIT_RATIO  0.4
@@ -193,11 +187,13 @@ struct _GdlDockItemMenuData {
 
 /* ----- Private functions ----- */
 
-GNOME_CLASS_BOILERPLATE (GdlDockItem, gdl_dock_item, GdlDockObject, GDL_TYPE_DOCK_OBJECT);
+GDL_CLASS_BOILERPLATE (GdlDockItem, gdl_dock_item, GdlDockObject, GDL_TYPE_DOCK_OBJECT);
 
 static void
 gdl_dock_item_class_init (GdlDockItemClass *klass)
 {
+    static gboolean style_initialized = FALSE;
+    
     GObjectClass       *g_object_class;
     GtkObjectClass     *gtk_object_class;
     GtkWidgetClass     *widget_class;
@@ -245,7 +241,7 @@ gdl_dock_item_class_init (GdlDockItemClass *klass)
         g_param_spec_enum ("orientation", _("Orientation"),
                            _("Orientation of the docking item"),
                            GTK_TYPE_ORIENTATION,
-                           GTK_ORIENTATION_HORIZONTAL,
+                           GTK_ORIENTATION_VERTICAL,
                            G_PARAM_READWRITE | G_PARAM_CONSTRUCT |
                            GDL_DOCK_PARAM_EXPORT));
 
@@ -273,13 +269,6 @@ gdl_dock_item_class_init (GdlDockItemClass *klass)
                             GDL_DOCK_ITEM_BEH_NORMAL,
                             G_PARAM_READWRITE));
                                      
-    g_object_class_install_property (
-        g_object_class, PROP_GRIP_SIZE,
-        g_param_spec_uint ("grip_size", _("Grip size"),
-                           _("Size in pixels of the grip to drag the dock item"),
-                           0, 100, DEFAULT_GRIP_SIZE,
-                           G_PARAM_READWRITE));
-
     g_object_class_install_property (
         g_object_class, PROP_LOCKED,
         g_param_spec_boolean ("locked", _("Locked"),
@@ -346,6 +335,18 @@ gdl_dock_item_class_init (GdlDockItemClass *klass)
     klass->dock_drag_motion = NULL;
     klass->dock_drag_end = NULL;
     klass->set_orientation = gdl_dock_item_real_set_orientation;
+
+    if (!style_initialized)
+    {
+        style_initialized = TRUE;
+        gtk_rc_parse_string (
+            "style \"gdl-dock-item-default\" {\n"
+            "xthickness = 0\n"
+            "ythickness = 0\n"
+            "}\n"
+            "class \"GdlDockItem\" "
+            "style : gtk \"gdl-dock-item-default\"\n");
+    }
 }
 
 static void
@@ -355,7 +356,7 @@ gdl_dock_item_instance_init (GdlDockItem *item)
 
     item->child = NULL;
     
-    item->orientation = GTK_ORIENTATION_HORIZONTAL;
+    item->orientation = GTK_ORIENTATION_VERTICAL;
     item->behavior = GDL_DOCK_ITEM_BEH_NORMAL;
 
     item->resize = TRUE;
@@ -366,7 +367,6 @@ gdl_dock_item_instance_init (GdlDockItem *item)
     item->_priv->menu = NULL;
 
     item->_priv->preferred_width = item->_priv->preferred_height = -1;
-    item->_priv->grip_size = DEFAULT_GRIP_SIZE;
     item->_priv->tab_label = NULL;
 
     item->_priv->ph = NULL;
@@ -379,7 +379,7 @@ gdl_dock_item_constructor (GType                  type,
 {
     GObject *g_object;
     
-    g_object = GNOME_CALL_PARENT_WITH_DEFAULT (G_OBJECT_CLASS, 
+    g_object = GDL_CALL_PARENT_WITH_DEFAULT (G_OBJECT_CLASS, 
                                                constructor, 
                                                (type,
                                                 n_construct_properties,
@@ -424,26 +424,34 @@ gdl_dock_item_set_property  (GObject      *g_object,
             item->behavior = g_value_get_flags (value);
 
             if ((old_beh ^ item->behavior) & GDL_DOCK_ITEM_BEH_LOCKED) {
+                if (GDL_DOCK_OBJECT_GET_MASTER (item))
+                    g_signal_emit_by_name (GDL_DOCK_OBJECT_GET_MASTER (item),
+                                           "layout_changed");
                 g_object_notify (g_object, "locked");
                 gdl_dock_item_showhide_grip (item);
             }
             
             break;
         }
-        case PROP_GRIP_SIZE:
-            item->_priv->grip_size = g_value_get_uint (value);
-            if (GDL_DOCK_ITEM_GRIP_SHOWN (item))
-                gtk_widget_queue_resize (GTK_WIDGET (item));
-            break;
         case PROP_LOCKED:
+        {
+            GdlDockItemBehavior old_beh = item->behavior;
+
             if (g_value_get_boolean (value))
                 item->behavior |= GDL_DOCK_ITEM_BEH_LOCKED;
             else
                 item->behavior &= ~GDL_DOCK_ITEM_BEH_LOCKED;
-            gdl_dock_item_showhide_grip (item);
-            g_object_notify (g_object, "behavior");
-            
+
+            if (old_beh ^ item->behavior) {
+		gdl_dock_item_showhide_grip (item);
+                g_object_notify (g_object, "behavior");
+
+                if (GDL_DOCK_OBJECT_GET_MASTER (item))
+                    g_signal_emit_by_name (GDL_DOCK_OBJECT_GET_MASTER (item),
+                                           "layout_changed");
+            }
             break;
+        }
         case PROP_PREFERRED_WIDTH:
             item->_priv->preferred_width = g_value_get_int (value);
             break;
@@ -474,9 +482,6 @@ gdl_dock_item_get_property  (GObject      *g_object,
         case PROP_BEHAVIOR:
             g_value_set_flags (value, item->behavior);
             break;
-        case PROP_GRIP_SIZE:
-            g_value_set_uint (value, item->_priv->grip_size);
-            break;
         case PROP_LOCKED:
             g_value_set_boolean (value, !GDL_DOCK_ITEM_NOT_LOCKED (item));
             break;
@@ -520,7 +525,7 @@ gdl_dock_item_destroy (GtkObject *object)
         g_free (priv);
     }
 
-    GNOME_CALL_PARENT (GTK_OBJECT_CLASS, destroy, (object));
+    GDL_CALL_PARENT (GTK_OBJECT_CLASS, destroy, (object));
 }
 
 static void 
@@ -620,6 +625,7 @@ gdl_dock_item_size_request (GtkWidget      *widget,
                             GtkRequisition *requisition)
 {
     GtkRequisition  child_requisition;
+    GtkRequisition  grip_requisition;
     GdlDockItem    *item;
 
     g_return_if_fail (GDL_IS_DOCK_ITEM (widget));
@@ -637,16 +643,26 @@ gdl_dock_item_size_request (GtkWidget      *widget,
     }
 
     if (item->orientation == GTK_ORIENTATION_HORIZONTAL) {
-        requisition->width = 
-            GDL_DOCK_ITEM_GRIP_SHOWN (item) ? item->_priv->grip_size : 0;
+        if (GDL_DOCK_ITEM_GRIP_SHOWN (item)) {
+            gtk_widget_size_request (item->_priv->grip, &grip_requisition);
+            requisition->width = grip_requisition.width;
+        } else {
+            requisition->width = 0;
+        }
+
         if (item->child) {
             requisition->width += child_requisition.width;
             requisition->height = child_requisition.height;
         } else
             requisition->height = 0;
     } else {
-        requisition->height = 
-            GDL_DOCK_ITEM_GRIP_SHOWN (item) ? item->_priv->grip_size : 0;
+        if (GDL_DOCK_ITEM_GRIP_SHOWN (item)) {
+            gtk_widget_size_request (item->_priv->grip, &grip_requisition);
+            requisition->height = grip_requisition.height;
+        } else {
+            requisition->height = 0;
+        }
+
         if (item->child) {
             requisition->width = child_requisition.width;
             requisition->height += child_requisition.height;
@@ -654,8 +670,8 @@ gdl_dock_item_size_request (GtkWidget      *widget,
             requisition->width = 0;
     }
 
-    requisition->width += GTK_CONTAINER (widget)->border_width * 2;
-    requisition->height += GTK_CONTAINER (widget)->border_width * 2;
+    requisition->width += (GTK_CONTAINER (widget)->border_width + widget->style->xthickness) * 2;
+    requisition->height += (GTK_CONTAINER (widget)->border_width + widget->style->ythickness) * 2;
 
     widget->requisition = *requisition;
 }
@@ -686,28 +702,31 @@ gdl_dock_item_size_allocate (GtkWidget     *widget,
 
         border_width = GTK_CONTAINER (widget)->border_width;
 
-        child_allocation.x = border_width;
-        child_allocation.y = border_width;
-        child_allocation.width = allocation->width - 2 * border_width;
-        child_allocation.height = allocation->height - 2 * border_width;
+        child_allocation.x = border_width + widget->style->xthickness;
+        child_allocation.y = border_width + widget->style->ythickness;
+        child_allocation.width = allocation->width
+            - 2 * (border_width + widget->style->xthickness);
+        child_allocation.height = allocation->height
+            - 2 * (border_width + widget->style->ythickness);
         
         if (GDL_DOCK_ITEM_GRIP_SHOWN (item)) {
-            GtkAllocation grip_alloc = *allocation;
+            GtkAllocation grip_alloc = child_allocation;
+            GtkRequisition grip_req;
             
-            grip_alloc.x = grip_alloc.y = 0;
+            gtk_widget_size_request (item->_priv->grip, &grip_req);
             
             if (item->orientation == GTK_ORIENTATION_HORIZONTAL) {
-                child_allocation.x += item->_priv->grip_size;
-                child_allocation.width -= item->_priv->grip_size;
-                grip_alloc.width = item->_priv->grip_size;
+                child_allocation.x += grip_req.width;
+                child_allocation.width -= grip_req.width;
+                grip_alloc.width = grip_req.width;
             } else {
-                child_allocation.y += item->_priv->grip_size;
-                child_allocation.height -= item->_priv->grip_size;
-                grip_alloc.height = item->_priv->grip_size;
+                child_allocation.y += grip_req.height;
+                child_allocation.height -= grip_req.height;
+                grip_alloc.height = grip_req.height;
             }
             if (item->_priv->grip)
                 gtk_widget_size_allocate (item->_priv->grip, &grip_alloc);
-        };
+        }
         gtk_widget_size_allocate (item->child, &child_allocation);
     }
 }
@@ -807,7 +826,6 @@ gdl_dock_item_style_set (GtkWidget *widget,
     g_return_if_fail (widget != NULL);
     g_return_if_fail (GDL_IS_DOCK_ITEM (widget));
 
-    /* FIXME: maybe remove this method altogether and use the default implementation */
     if (GTK_WIDGET_REALIZED (widget) && !GTK_WIDGET_NO_WINDOW (widget)) {
         gtk_style_set_background (widget->style, widget->window,
                                   widget->state);
@@ -843,12 +861,15 @@ gdl_dock_item_expose (GtkWidget      *widget,
 
     if (GTK_WIDGET_DRAWABLE (widget) && event->window == widget->window) {
         gdl_dock_item_paint (widget, event);
-        GNOME_CALL_PARENT (GTK_WIDGET_CLASS, expose_event, (widget, event));
+        GDL_CALL_PARENT (GTK_WIDGET_CLASS, expose_event, (widget, event));
     }
   
     return FALSE;
 }
 
+#define EVENT_IN_GRIP_EVENT_WINDOW(ev,gr) \
+    ((gr) != NULL && (ev)->window == GDL_DOCK_ITEM_GRIP (gr)->title_window)
+
 #define EVENT_IN_TABLABEL_EVENT_WINDOW(ev,tl) \
     ((tl) != NULL && (ev)->window == GDL_DOCK_TABLABEL (tl)->event_window)
 
@@ -857,32 +878,31 @@ gdl_dock_item_button_changed (GtkWidget      *widget,
                               GdkEventButton *event)
 {
     GdlDockItem *item;
+    gboolean     locked;
     gboolean     event_handled;
     gboolean     in_handle;
+    GdkCursor   *cursor;
   
     g_return_val_if_fail (widget != NULL, FALSE);
     g_return_val_if_fail (GDL_IS_DOCK_ITEM (widget), FALSE);
     g_return_val_if_fail (event != NULL, FALSE);
     
     item = GDL_DOCK_ITEM (widget);
-    
-    if (!(event->window == widget->window ||
-          EVENT_IN_TABLABEL_EVENT_WINDOW (event, item->_priv->tab_label)))
+
+    if (!EVENT_IN_GRIP_EVENT_WINDOW (event, item->_priv->grip))
         return FALSE;
     
-    /* Verify that the item is not locked. */
-    if (!GDL_DOCK_ITEM_NOT_LOCKED (item))
-        return FALSE;
+    locked = !GDL_DOCK_ITEM_NOT_LOCKED (item);
 
     event_handled = FALSE;
 
     /* Check if user clicked on the drag handle. */      
     switch (item->orientation) {
     case GTK_ORIENTATION_HORIZONTAL:
-        in_handle = event->x < item->_priv->grip_size;
+        in_handle = event->x < item->_priv->grip->allocation.width;
         break;
     case GTK_ORIENTATION_VERTICAL:
-        in_handle = event->y < item->_priv->grip_size;
+        in_handle = event->y < item->_priv->grip->allocation.height;
         break;
     default:
         in_handle = FALSE;
@@ -890,18 +910,24 @@ gdl_dock_item_button_changed (GtkWidget      *widget,
     }
 
     /* Left mousebutton click on dockitem. */
-    if (event->button == 1 && event->type == GDK_BUTTON_PRESS) {
+    if (!locked && event->button == 1 && event->type == GDK_BUTTON_PRESS) {
         /* Set in_drag flag, grab pointer and call begin drag operation. */      
         if (in_handle) {
             item->_priv->start_x = event->x;
             item->_priv->start_y = event->y;
 
             GDL_DOCK_ITEM_SET_FLAGS (item, GDL_DOCK_IN_PREDRAG);
+            
+            cursor = gdk_cursor_new_for_display (gtk_widget_get_display (widget),
+                                                 GDK_FLEUR);
+            gdk_window_set_cursor (GDL_DOCK_ITEM_GRIP (item->_priv->grip)->title_window,
+                                   cursor);
+            gdk_cursor_unref (cursor);
         
             event_handled = TRUE;
         };
         
-    } else if (event->type == GDK_BUTTON_RELEASE && event->button == 1) {
+    } else if (!locked &&event->type == GDK_BUTTON_RELEASE && event->button == 1) {
         if (GDL_DOCK_ITEM_IN_DRAG (item)) {
             /* User dropped widget somewhere. */
             gdl_dock_item_drag_end (item, FALSE);
@@ -911,7 +937,17 @@ gdl_dock_item_button_changed (GtkWidget      *widget,
             GDL_DOCK_ITEM_UNSET_FLAGS (item, GDL_DOCK_IN_PREDRAG);
             event_handled = TRUE;
         }
-             
+
+        /* we check the window since if the item was redocked it's
+           been unrealized and maybe it's not realized again yet */
+        if (GDL_DOCK_ITEM_GRIP (item->_priv->grip)->title_window) {
+            cursor = gdk_cursor_new_for_display (gtk_widget_get_display (widget),
+                                                 GDK_HAND2);
+            gdk_window_set_cursor (GDL_DOCK_ITEM_GRIP (item->_priv->grip)->title_window,
+                                   cursor);
+            gdk_cursor_unref (cursor);
+        }
+
     } else if (event->button == 3 && event->type == GDK_BUTTON_PRESS && in_handle) {
         gdl_dock_item_popup_menu (item, event->button, event->time);
         event_handled = TRUE;    	
@@ -933,8 +969,7 @@ gdl_dock_item_motion (GtkWidget      *widget,
 
     item = GDL_DOCK_ITEM (widget);
 
-    if (!(event->window == widget->window ||
-          EVENT_IN_TABLABEL_EVENT_WINDOW (event, item->_priv->tab_label)))
+    if (!EVENT_IN_GRIP_EVENT_WINDOW (event, item->_priv->grip))
         return FALSE;
 
     if (GDL_DOCK_ITEM_IN_PREDRAG (item)) {
@@ -979,7 +1014,7 @@ gdl_dock_item_key_press (GtkWidget   *widget,
     if (event_handled)
         return TRUE;
     else
-        return GNOME_CALL_PARENT_WITH_DEFAULT (GTK_WIDGET_CLASS,
+        return GDL_CALL_PARENT_WITH_DEFAULT (GTK_WIDGET_CLASS,
                                                key_press_event,
                                                (widget, event),
                                                FALSE);
@@ -1197,52 +1232,10 @@ static void
 gdl_dock_item_detach_menu (GtkWidget *widget,
                            GtkMenu   *menu)
 {
-    GdlDockItem         *item;
-    GdlDockItemMenuData *menu_data;
+    GdlDockItem *item;
    
     item = GDL_DOCK_ITEM (widget);
-    menu_data = g_object_get_data (G_OBJECT (menu), "user_data");
-
-    /* destroy menu items */
-    gtk_widget_unref (menu_data->dock);
-    gtk_widget_unref (menu_data->undock);
-    gtk_widget_destroy (menu_data->dock);
-    gtk_widget_destroy (menu_data->undock);
-
-    /* release menu data struct */
     item->_priv->menu = NULL;
-    g_free (menu_data);
-    g_object_set_data (G_OBJECT (menu), "user_data", NULL);
-}
-
-static void
-gdl_dock_item_dock_cb (GtkWidget   *widget,
-                       GdlDockItem *item)
-{
-#if 0
-    g_return_if_fail (item != NULL);
-    /* force docking even if saved position is floating */
-    if (item->last_pos.position == GDL_DOCK_FLOATING)
-        item->last_pos.position = GDL_DOCK_TOP;
-
-    gdl_dock_item_restore_position (item);
-
-    /* layout has changed */
-    g_signal_emit_by_name (item->dock, "layout_changed");
-#endif
-}
-
-static void
-gdl_dock_item_undock_cb (GtkWidget   *widget,
-                         GdlDockItem *item)
-{
-#if 0
-    g_return_if_fail (item != NULL);
-
-    /* current position is saved in dock_to when the item floats */
-    gdl_dock_item_dock_to (item, NULL, GDL_DOCK_FLOATING, -1);
-    g_signal_emit_by_name (item->dock, "layout_changed");
-#endif
 }
 
 static void
@@ -1251,63 +1244,35 @@ gdl_dock_item_popup_menu (GdlDockItem  *item,
                           guint32       time)
 {
     GtkWidget *mitem;
-    GdlDockItemMenuData *menu_data;
 
     if (!item->_priv->menu) {
         /* Create popup menu and attach it to the dock item */
         item->_priv->menu = gtk_menu_new ();
-
-        menu_data = g_new0 (GdlDockItemMenuData, 1);
-        g_object_set_data (G_OBJECT (item->_priv->menu), "user_data", menu_data);
-        gtk_menu_attach_to_widget (GTK_MENU (item->_priv->menu), 
+        gtk_menu_attach_to_widget (GTK_MENU (item->_priv->menu),
                                    GTK_WIDGET (item),
                                    gdl_dock_item_detach_menu);
         
-        /* Dock/Undock menuitem. */
-        menu_data->dock = gtk_menu_item_new_with_label (_("Dock"));
-        menu_data->undock = gtk_menu_item_new_with_label (_("Undock"));
-        g_signal_connect (menu_data->dock, "activate", 
-                          G_CALLBACK (gdl_dock_item_dock_cb), item);
-        g_signal_connect (menu_data->undock, "activate", 
-                          G_CALLBACK (gdl_dock_item_undock_cb), item);
-        gtk_widget_ref (menu_data->dock);
-        gtk_widget_ref (menu_data->undock);
-        gtk_object_sink (GTK_OBJECT (menu_data->dock));
-        gtk_object_sink (GTK_OBJECT (menu_data->undock));
-        menu_data->first_option = NULL;
-
-        /* Hide menuitem. */
-        mitem = menu_data->hide = gtk_menu_item_new_with_label (_("Hide"));
-        gtk_menu_shell_append (GTK_MENU_SHELL (item->_priv->menu), menu_data->hide);
-        g_signal_connect (mitem, "activate", 
-                          G_CALLBACK (gdl_dock_item_hide_cb), item);
-
-        /* Lock menuitem */
-        mitem = menu_data->lock = gtk_menu_item_new_with_label (_("Lock"));
-        gtk_menu_shell_append (GTK_MENU_SHELL (item->_priv->menu), menu_data->lock);
-        g_signal_connect (mitem, "activate",
-                          G_CALLBACK (gdl_dock_item_lock_cb), item);
-
-    } else
-        menu_data = g_object_get_data (G_OBJECT (item->_priv->menu), "user_data");
-
-    /* update menu options */
-    if (menu_data->first_option) {
-        gtk_container_remove (GTK_CONTAINER (item->_priv->menu), 
-                              menu_data->first_option);
-        menu_data->first_option = NULL;
+	if (item->behavior & GDL_DOCK_ITEM_BEH_LOCKED) {
+            /* UnLock menuitem */
+            mitem = gtk_menu_item_new_with_label (_("UnLock"));
+            gtk_menu_shell_append (GTK_MENU_SHELL (item->_priv->menu), 
+			           mitem);
+            g_signal_connect (mitem, "activate",
+                              G_CALLBACK (gdl_dock_item_unlock_cb), item);
+	} else {
+            /* Hide menuitem. */
+            mitem = gtk_menu_item_new_with_label (_("Hide"));
+            gtk_menu_shell_append (GTK_MENU_SHELL (item->_priv->menu), mitem);
+            g_signal_connect (mitem, "activate", 
+                              G_CALLBACK (gdl_dock_item_hide_cb), item);
+	    /* Lock menuitem */
+            mitem = gtk_menu_item_new_with_label (_("Lock"));
+            gtk_menu_shell_append (GTK_MENU_SHELL (item->_priv->menu), mitem);
+            g_signal_connect (mitem, "activate",
+                              G_CALLBACK (gdl_dock_item_lock_cb), item);
+	}
     }
 
-#if 0
-    if (GDL_DOCK_ITEM_IS_FLOATING (item)) {
-        gtk_menu_shell_prepend (GTK_MENU_SHELL (item->menu), menu_data->dock);
-        menu_data->first_option = menu_data->dock;
-    } else {
-        gtk_menu_shell_prepend (GTK_MENU_SHELL (item->menu), menu_data->undock);
-        menu_data->first_option = menu_data->undock;
-    };
-#endif
-
     /* Show popup menu. */
     gtk_widget_show_all (item->_priv->menu);
     gtk_menu_popup (GTK_MENU (item->_priv->menu), NULL, NULL, NULL, NULL, 
@@ -1365,11 +1330,11 @@ gdl_dock_item_tab_button (GtkWidget      *widget,
            drag handle */
         switch (item->orientation) {
         case GTK_ORIENTATION_HORIZONTAL:
-            item->dragoff_x = item->_priv->grip_size / 2;
+            /*item->dragoff_x = item->_priv->grip_size / 2;*/
             item->dragoff_y = GTK_WIDGET (data)->allocation.height / 2;
             break;
         case GTK_ORIENTATION_VERTICAL:
-            item->dragoff_x = GTK_WIDGET (data)->allocation.width / 2;
+            /*item->dragoff_x = GTK_WIDGET (data)->allocation.width / 2;*/
             item->dragoff_y = item->_priv->grip_size / 2;
             break;
         };
@@ -1395,8 +1360,6 @@ gdl_dock_item_hide_cb (GtkWidget   *widget,
 
     master = GDL_DOCK_OBJECT_GET_MASTER (item);
     gdl_dock_item_hide_item (item);
-    if (master)
-        g_signal_emit_by_name (master, "layout_changed");
 }
 
 static void
@@ -1406,21 +1369,38 @@ gdl_dock_item_lock_cb (GtkWidget   *widget,
     g_return_if_fail (item != NULL);
 
     gdl_dock_item_lock (item);
-    g_signal_emit_by_name (GDL_DOCK_OBJECT_GET_MASTER (item), "layout_changed");
+}
+
+static void
+gdl_dock_item_unlock_cb (GtkWidget   *widget,
+                       GdlDockItem *item)
+{
+    g_return_if_fail (item != NULL);
+
+    gdl_dock_item_unlock (item);
 }
 
 static void
 gdl_dock_item_showhide_grip (GdlDockItem *item)
 {
+    GdkDisplay *display;
+    GdkCursor *cursor;
+    
+    gdl_dock_item_detach_menu (GTK_WIDGET (item), NULL); 
+    display = gtk_widget_get_display (GTK_WIDGET (item));
+    cursor = NULL;
+    
     if (item->_priv->grip) {
-        if (GDL_DOCK_ITEM_GRIP_SHOWN (item)) {
-            gtk_widget_show (item->_priv->grip);
-            GTK_WIDGET_SET_FLAGS (item->_priv->grip, GTK_CAN_FOCUS);
-        } else {
-            gtk_widget_hide (item->_priv->grip);
-            GTK_WIDGET_UNSET_FLAGS (item->_priv->grip, GTK_CAN_FOCUS);
-        }
+        if (GDL_DOCK_ITEM_GRIP_SHOWN (item) && 
+	    GDL_DOCK_ITEM_NOT_LOCKED(item))
+             cursor = gdk_cursor_new_for_display (display, GDK_HAND2);
     }
+    if (item->_priv->grip && GDL_DOCK_ITEM_GRIP (item->_priv->grip)->title_window)
+        gdk_window_set_cursor (GDL_DOCK_ITEM_GRIP (item->_priv->grip)->title_window, cursor);
+
+    if (cursor)
+        gdk_cursor_unref (cursor);
+    
     gtk_widget_queue_resize (GTK_WIDGET (item));
 }
 
@@ -1451,7 +1431,27 @@ gdl_dock_item_new (const gchar         *name,
                                         "behavior", behavior,
                                         NULL));
     GDL_DOCK_OBJECT_UNSET_FLAGS (item, GDL_DOCK_AUTOMATIC);
-    gdl_dock_item_set_tablabel (item, gdl_dock_tablabel_new (item));
+    gdl_dock_item_set_tablabel (item, gtk_label_new (long_name));
+
+    return GTK_WIDGET (item);
+}
+
+GtkWidget *
+gdl_dock_item_new_with_stock (const gchar         *name,
+                              const gchar         *long_name,
+                              const gchar         *stock_id,
+                              GdlDockItemBehavior  behavior)
+{
+    GdlDockItem *item;
+
+    item = GDL_DOCK_ITEM (g_object_new (GDL_TYPE_DOCK_ITEM, 
+                                        "name", name, 
+                                        "long_name", long_name,
+                                        "stock_id", stock_id,
+                                        "behavior", behavior,
+                                        NULL));
+    GDL_DOCK_OBJECT_UNSET_FLAGS (item, GDL_DOCK_AUTOMATIC);
+    gdl_dock_item_set_tablabel (item, gtk_label_new (long_name));
 
     return GTK_WIDGET (item);
 }
@@ -1564,6 +1564,7 @@ gdl_dock_item_hide_grip (GdlDockItem *item)
         item->_priv->grip_shown = FALSE;
         gdl_dock_item_showhide_grip (item);
     };
+    g_warning ("Grips always show unless GDL_DOCK_ITEM_BEH_NO_GRIP is set\n" );
 }
 
 void
@@ -1605,7 +1606,7 @@ gdl_dock_item_hide_item (GdlDockItem *item)
     if (!GDL_DOCK_OBJECT_ATTACHED (item))
         /* already hidden/detached */
         return;
-    
+       
     /* if the object is manual, create a new placeholder to be able to
        restore the position later */
     if (!GDL_DOCK_OBJECT_AUTOMATIC (item)) {
@@ -1632,14 +1633,27 @@ gdl_dock_item_hide_item (GdlDockItem *item)
     /* detach the item recursively */
     gdl_dock_object_detach (GDL_DOCK_OBJECT (item), TRUE);
 
+    gtk_widget_hide (GTK_WIDGET (item));
+	
     gdl_dock_object_thaw (GDL_DOCK_OBJECT (item));
 }
 
+void
+gdl_dock_item_iconify_item (GdlDockItem *item)
+{
+    g_return_if_fail (item != NULL);
+    
+    GDL_DOCK_OBJECT_SET_FLAGS (item, GDL_DOCK_ICONIFIED);
+    gdl_dock_item_hide_item (item);
+}
+
 void
 gdl_dock_item_show_item (GdlDockItem *item)
 {
     g_return_if_fail (item != NULL);
 
+    GDL_DOCK_OBJECT_UNSET_FLAGS (item, GDL_DOCK_ICONIFIED);
+
     if (item->_priv->ph) {
         gtk_container_add (GTK_CONTAINER (item->_priv->ph), GTK_WIDGET (item));
         g_object_unref (item->_priv->ph);
@@ -1653,6 +1667,7 @@ gdl_dock_item_show_item (GdlDockItem *item)
                                   GDL_DOCK_FLOATING, NULL);
         }
     }
+    gtk_widget_show (GTK_WIDGET (item));
 }
 
 void
