@@ -6,20 +6,19 @@
  *
  * Copyright (C) 2002 Gustavo Girldez <gustavo.giraldez@gmx.net>
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.  
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #ifdef HAVE_CONFIG_H
@@ -27,7 +26,6 @@
 #endif
 
 #include "gdl-i18n.h"
-#include <libgnome/gnome-macros.h>
 #include <stdlib.h>
 #include <string.h>
 
@@ -82,6 +80,7 @@ enum {
     PROP_0,
     PROP_NAME,
     PROP_LONG_NAME,
+    PROP_STOCK_ID,
     PROP_MASTER,
     PROP_EXPORT_PROPERTIES
 };
@@ -96,7 +95,7 @@ static guint gdl_dock_object_signals [LAST_SIGNAL] = { 0 };
 
 /* ----- Private interface ----- */
 
-GNOME_CLASS_BOILERPLATE (GdlDockObject, gdl_dock_object, GtkContainer, GTK_TYPE_CONTAINER);
+GDL_CLASS_BOILERPLATE (GdlDockObject, gdl_dock_object, GtkContainer, GTK_TYPE_CONTAINER);
 
 static void
 gdl_dock_object_class_init (GdlDockObjectClass *klass)
@@ -130,6 +129,13 @@ gdl_dock_object_class_init (GdlDockObjectClass *klass)
                              NULL,
                              G_PARAM_READWRITE));
 
+    g_object_class_install_property (
+        g_object_class, PROP_STOCK_ID,
+        g_param_spec_string ("stock_id", _("Stock Icon"),
+                             _("Stock icon for the dock object"),
+                             NULL,
+                             G_PARAM_READWRITE));
+
     g_object_class_install_property (
         g_object_class, PROP_MASTER,
         g_param_spec_object ("master", _("Dock master"),
@@ -203,6 +209,10 @@ gdl_dock_object_set_property  (GObject      *g_object,
         g_free (object->long_name);
         object->long_name = g_value_dup_string (value);
         break;
+    case PROP_STOCK_ID:
+        g_free (object->stock_id);
+        object->stock_id = g_value_dup_string (value);
+        break;
     case PROP_MASTER:
         if (g_value_get_object (value)) 
             gdl_dock_object_bind (object, g_value_get_object (value));
@@ -230,6 +240,9 @@ gdl_dock_object_get_property  (GObject      *g_object,
     case PROP_LONG_NAME:
         g_value_set_string (value, object->long_name);
         break;
+    case PROP_STOCK_ID:
+        g_value_set_string (value, object->stock_id);
+        break;
     case PROP_MASTER:
         g_value_set_object (value, object->master);
         break;
@@ -252,8 +265,10 @@ gdl_dock_object_finalize (GObject *g_object)
     object->name = NULL;
     g_free (object->long_name);
     object->long_name = NULL;
+    g_free (object->stock_id);
+    object->stock_id = NULL;
 
-    GNOME_CALL_PARENT (G_OBJECT_CLASS, finalize, (g_object));
+    GDL_CALL_PARENT (G_OBJECT_CLASS, finalize, (g_object));
 }
 
 static void
@@ -290,7 +305,7 @@ gdl_dock_object_destroy (GtkObject *gtk_object)
     if (object->master)
         gdl_dock_object_unbind (object);
         
-    GNOME_CALL_PARENT (GTK_OBJECT_CLASS, destroy, (gtk_object));
+    GDL_CALL_PARENT (GTK_OBJECT_CLASS, destroy, (gtk_object));
 }
 
 static void
@@ -311,7 +326,7 @@ gdl_dock_object_show (GtkWidget *widget)
                                (GtkCallback) gdl_dock_object_foreach_automatic,
                                gtk_widget_show);
     }
-    GNOME_CALL_PARENT (GTK_WIDGET_CLASS, show, (widget));
+    GDL_CALL_PARENT (GTK_WIDGET_CLASS, show, (widget));
 }
 
 static void
@@ -322,7 +337,7 @@ gdl_dock_object_hide (GtkWidget *widget)
                                (GtkCallback) gdl_dock_object_foreach_automatic,
                                gtk_widget_hide);
     }
-    GNOME_CALL_PARENT (GTK_WIDGET_CLASS, hide, (widget));
+    GDL_CALL_PARENT (GTK_WIDGET_CLASS, hide, (widget));
 }
 
 static void
@@ -338,7 +353,7 @@ gdl_dock_object_real_detach (GdlDockObject *object,
     if (recursive && gdl_dock_object_is_compound (object)) {
         gtk_container_foreach (GTK_CONTAINER (object),
                                (GtkCallback) gdl_dock_object_detach,
-                               (gpointer) recursive);
+                               GINT_TO_POINTER (recursive));
     }
     
     /* detach the object itself */
